/* Ovaj parser gradimo koristeci viseznacnu gramatiku za aritmeticke izraze */

%{

#include <stdio.h>
#include <stdlib.h>

/* Ukoliko zelimo da koristimo debagovanje i pratimo stanja parsera neophodno je da definisemo YYDEBUG i u main() funkciji da 
	izvrsimo dodelu yydebug =1 */
#define YYDEBUG 1

/* Leksicki analizator */ 
extern int yylex();

/* Funkcija za obradu greske */
int yyerror(char *s){
	fprintf(stderr,"%s\n",s);
	exit(EXIT_FAILURE);	
}
%}

/* Tip atributa simbola ce biti tip nekog od polja ove unije. */
%union{
	float vrednost;
}

/* Definisemo asocijativnost i prioritete operatora. */
%left '+' '-'
%left '*' '/' 
%left UMINUS

/* Pridruzujemo konkretno polje unije tokenu i neterminalu e */
%type <vrednost> e
/* tokenu dodeljujemo atribut */
%token <vrednost> BROJ
/* zadajemo aksiomu gramatike */
%start pocetak
%%
/* pravila KSG sa pridruzenim semantickim akcijama */
pocetak: e 			{ printf("Vrednost izraza je %.2f\n", $1);}
	;
e : e '+' e  		{ $$  = $1 +$3;}
	| e '*' e		{ $$ = $1 * $3;}
	|  e '-' e		{ $$ = $1 - $3;}
	|  e '/' e		{ if( $3 == 0){
							yyerror("Deljenje nulom!");	
						} 
					  $$ = $1 / $3; }
/* Dodeljujemo pravilu prioritet. Koristeci debagovanje proveriti redosled primenjivanja pravila kada nije zadat prioritet i kada je prioritet dodeljen. */
	| '-' e %prec UMINUS	{ $$ = -$2; }
	| '(' e ')'     { $$ = $2; }
	| BROJ			{ $$ = $1;}
	; 
%%

int main(){
	/* ukoliko ne zelimo debagovanje onda je potrebno postaviti vrednost od yydebug na 0 */
	yydebug = 0;
	yyparse();
	return 0;
}
